# coding: utf-8
"""
    AtsasTools.py

    Copyright (c) 2016-2017, Masatsuyo Takahashi, KEK-PF
"""
import sys
import os
import glob
import re
import numpy                as np
from subprocess             import Popen, PIPE
import codecs
import struct
from IntensityData          import IntensityData
from Guinier                import Guinier
from WLS_Rg                 import WLS_Rg, OLS_Rg
from Result                 import Result
from Settings               import get_setting, set_setting

autorg_exe_array = []

def set_exe_array():
    atsas_dir = glob.glob( 'C:/atsas*' )
    for dir_ in atsas_dir:
        for sub_dir in [ '/bin', '' ]:
            exe_path = dir_ + sub_dir + '/autorg.exe'
            if os.path.exists( exe_path ):
                autorg_exe_array.append( exe_path )
                break

    atsas_exe_paths = get_setting( 'atsas_exe_paths' )
    if len( autorg_exe_array ) > 0:
        set_setting( 'atsas_exe_paths', autorg_exe_array )

    # print( 'autorg_exe_array=', autorg_exe_array )

set_exe_array()

def double_to_float( d ):
    # http://stackoverflow.com/questions/13291539/convert-double-to-float-in-python
    # return struct.unpack("f", struct.pack("f", d))
    return float( '%g' % d )

def autorg( file, exe_index=0, out_file=None, smaxrg=1.3 ):
    # print( 'autorg: file=', file )
    assert( os.path.exists( file ) )

    exe_ = autorg_exe_array[ exe_index ]
    opts = ' --smaxrg ' + str(smaxrg)

    if out_file is not None:
        opts += ' -o ' + out_file

    cmd = '%s %s %s -f csv' % ( exe_, opts, file )
    p = Popen( cmd, shell=True, stdout=PIPE, stderr=PIPE )
    out, err = p.communicate()
    # print( out, err )
    if err == b'':
        lines = out.split( b'\r\n' )
        if len( lines ) >= 3:
            intensity = IntensityData( file )
            x, y, e = intensity.get_guinier_valid_xy()

            File, R, RSD, I, ISD, F, L, Q, A = lines[1].split( b',' )
            print( R, RSD, I, ISD, F, L, Q, A )
            f, t = int(F)-1, int(L)-1
            f_, t_ = list( map( lambda i: intensity.positive_index[i], ( f, t ) ) )
            Rg = float( R )
            minQ = np.sqrt( x[f_] )
            maxQ = np.sqrt( x[t_] )

            orig_result = Result(
                            type='A',
                            Rg=Rg, Rg_stdev=float( RSD ),
                            I0=float(I), I0_stdev=float(ISD),
                            From=f_, To=t_,
                            Quality=float(Q),
                            Aggregated=float(A),
                            min_qRg=minQ * Rg,
                            max_qRg=maxQ * Rg,
                            min_curvature=None,
                            max_curvature=None,
                            )

            guinier = Guinier( x, y, e, positive_ratio=intensity.positive_ratio )
            guinier.estimate_rg( [ f_, t_ ], [ 0, 1.3 ] )
            eval_result = guinier.get_result()

            return orig_result, eval_result
        else:
            return None, None

    else:
        print( codecs.decode( err, 'cp932' ) )
        return None, None

class AlmergeExecutor:
    def __init__( self, c_vector, datafiles ):
        self.exe_path   = None
        if len( autorg_exe_array ) > 0:
            almerge_path = autorg_exe_array[0].replace( 'autorg.exe', 'almerge.exe' )
            if os.path.exists( almerge_path ):
                self.exe_path   = almerge_path

        self.datafiles  = datafiles
        self.c_vector   = c_vector
        self.out_line_re    = re.compile( r'(\d+)\s*-\s*(\d+)' )

    def execute( self, indeces, out_file ):

        # print( 'indeces=', indeces )
        # print( 'c_vector=', self.c_vector[indeces] )

        cmd_args = ' '.join( [ '-c %g %s' % ( self.c_vector[i],  self.datafiles[i] ) 
                                for i in indeces ] )

        cmd = '%s %s -z -o %s' % ( self.exe_path, cmd_args, out_file )
        p = Popen( cmd, shell=True, stdout=PIPE, stderr=PIPE )
        out, err = p.communicate()

        out = out.decode( 'cp932' ).replace( r'\r\n', '\n' ).replace( r'\\', r'/' )
        err = err.decode( 'cp932' )

        print( out )
        if err != '':
            print( err )

        if err != '': return None

        out_lines = out.split( '\n' )
        overlap_from_max = None
        for line in out_lines:
            m = self.out_line_re.search( line )
            if m:
                # print( m.group(1), m.group(2) )
                overlap_from = int( m.group(1) )
                if overlap_from_max is None or overlap_from > overlap_from_max:
                    overlap_from_max = overlap_from

        if not os.path.exists( out_file ):
            # TODO: error info
            return None

        fh = open( out_file )

        # generator to read the result file
        def read_data_lines( fh ):
            dataline_re = re.compile( r'^\s*\d' )
            while True:
                line = fh.readline()
                if not line:
                    break
                if dataline_re.match( line ):
                    yield line
                else:
                    continue

        exz_array = np.loadtxt( read_data_lines(fh) )

        return Result( exz_array=exz_array, overlap_from_max=overlap_from_max )
