# coding: utf-8
"""

    ファイル名：   Settings.py

    処理内容：

        設定情報

    Copyright (c) 2016-2017, Masatsuyo Takahashi, KEK-PF
"""
from __future__ import division, print_function, unicode_literals

from PersistentInfo     import PersistentInfo
from AppVersion         import get_app_version_string

# from MachineTypes       import get_display_resolution, get_chassistype_name, get_monitors

# TODO: import wmi into exe
# width, height = get_display_resolution()
# print( 'display width, height=', width, height )

# chassistype_name = get_chassistype_name()
# print( chassistype_name )

# TODO: threading because it takes a significant amountt of time
# for monitor in get_monitors():
#    print( 'monitor=', monitor )
"""
if chassistype_name == 'Desktop':
    default_main_geometry = '1080x750'
else:
    default_main_geometry = '1080x700'
"""
default_main_geometry = '1080x500'
# 永続的な記憶をオブジェクト化する（メモリに取り込む）。
setting_file    = 'settings.dump'
setting_info    = PersistentInfo( setting_file )
settings_       = setting_info.get_dictionary()
mask_           = None

stamp_name = 'version_stamp'

ITEM_DEFAULTS = {
    stamp_name              : None,
    'in_folder'             : None,
    'uv_folder'             : None,
    'uv_file'               : None,
    'an_folder'             : None,
    'analysis_name'         : 'analysis-000',
    'analysis_folder'       : None,
    'auto_number'           : 1,
    'save_averaged_data'    : 1,
    'averaged_data_folder'  : None,
    'file_extension'        : '*.dat',
    'atsas_exe_paths'       : [],
    'num_recent_folders'    : 20,
    'recent_folders'        : {},
    'conc_adjust'           : 0,
    'conc_adjust_amount'    : None,
    'mapping_show_mode'     : 'locally',
    'oopt_fit_consistency'  : 1,
    'oopt_optimize_only'    : 0,
    'oopt_qrg_limits'       : 'Limited',
    'oopt_qrg_limits_vals'  : [ 0.0, 1.3 ],
    'oopt_qrg_limits_apply' : 1,
    'min_quality'           : 0.5,
    'min_conc'              : 0.5,
    'serial_proc_cb_array'  : [ 1, 1, 0 ],
    'range_type'            : 3,        # Mixed Strategy
    'num_ranges'            : 1,
    'range_0_select'        : 1,
    'range_0_start'         : None,
    'range_0_peak'          : None,
    'range_0_end'           : None,
    'range_1_select'        : 1,
    'range_1_start'         : None,
    'range_1_peak'          : None,
    'range_1_end'           : None,
    'range_2_select'        : 1,
    'range_2_start'         : None,
    'range_2_peak'          : None,
    'range_2_end'           : None,
    'input_smoothing'       : 1,
    'input_smoothing_timing' : 1,
    'input_smoothing_method' : 2,
    'num_partitions_gpr'    : 200,
    'num_curves_averaged'   : 5,
    'atsas_exz_result'      : 1,
    'output_smoothing'      : 0,
    'result_book'           : 'analysis_report.xlsx',
    'force_recomp'          : 0,
    'force_recomp_temp'     : 0,
    'extinction'            : 1,
    'path_length'           : 5,
    'absorbance_picking'    : 280,
    'zero_absorbance'       : 340,
    'consider_scatter'      : 0,
    'scatter_picking'       : None,
    'num_points_absorbance' : 1,
    'intensity_picking'     : 0.020,
    'intensity_slice'       : '50:60',
    'num_points_intensity'  : 10,
    'quality_weighting'     : [
                                0.2,    # basic_quality
                                0.2,    # positive_score
                                0.0,    # fit_consistency
                                0.3,    # rg_stdev_score
                                0.3,    # q_rg_score
                              ],
    'locally_2d_regression' : True,
    'zx_reg_method'         : '3P',
    'zx_build_method'       : 'MAX',
    'zx_boundary_method'    : 'AUTO',
    'zx_boundary'           : 0.1,
    'zx_a_file'             : 'zx_A_file.dat',
    'zx_b_file'             : 'zx_B_file.dat',
    'zx_save_data'          : 0,
    'zx_out_folder'         : None,

    'intensity_reduction'   : 0,
    'reduction_method'      : 'THIN-OUT',
    'reduction_cycle'       : 4,
    'reduction_start'       : 0,
    }

TEMPORARY_ITEMS = [
    'uv_folder', 'uv_file',
    'force_recomp_temp', 'averaged_data_folder',
    'zx_out_folder',
    'conc_adjust', 'conc_adjust_amount',
    'intensity_reduction',
    'num_ranges', 
    'range_0_select', 'range_0_start', 'range_0_peak', 'range_0_end',
    'range_1_select', 'range_1_start', 'range_1_peak', 'range_1_end',
    'range_2_select', 'range_2_start', 'range_2_peak', 'range_2_end',
    ]

ALTERNATIVE_WEIGHTS = [ 0.2, 0.2, 0.3, 0.2, 0.1 ]

def reload_settings():
    global setting_info
    global settings_

    setting_info    = PersistentInfo( setting_file )
    settings_       = setting_info.get_dictionary()

def clear_settings():
    global settings_
    settings_ = {}
    setting_info.set_dictionary( settings_ )

def get_setting( item ):
    assert( item in ITEM_DEFAULTS )
    value = settings_.get( item )
    if value is None:
        value = ITEM_DEFAULTS.get( item )
        set_setting( item, value )
    return value

def set_setting( item, value ):
    assert( item in ITEM_DEFAULTS )
    settings_[ item ] = value

def temporary_settings_begin():
    global settings_save
    settings_save = settings_.copy()

def temporary_settings_end():
    global settings_save
    global settings_
    settings_ = settings_save
    setting_info.set_dictionary( settings_ )

def save_settings():
    setting_info.save()

def clear_temporary_settings():
    for item in TEMPORARY_ITEMS:
        set_setting( item, ITEM_DEFAULTS[item] )

def check_version_stamp():
    version_stamp = get_setting( stamp_name )
    if version_stamp is None or version_stamp != get_app_version_string():
        # TODO: MessageBox
        clear_settings()
    set_setting( stamp_name, get_app_version_string() )

def initialize():
    check_version_stamp()

initialize()
