import numpy as np
import math
import statsmodels.api      as sm
from scipy.interpolate      import UnivariateSpline

class Smoother:
    def __init__( self, x, y, frac=0.25 ):
        self.x  = x
        self.y  = y
        self.lowess = sm.nonparametric.lowess(y, x, frac=frac)
        self.smoother = UnivariateSpline( self.lowess[:,0], self.lowess[:,1], k=5 )

    def __call__( self, x ):
        return self.smoother( x )

if __name__ == '__main__':
    import matplotlib.pyplot    as plt
    n = 100
    x = np.linspace(0, 2 * math.pi, n)
    y = np.sin(x) + 0.3 * np.random.randn(n)

    f = 0.25
    lowess = sm.nonparametric.lowess(y, x, frac=f)
    lowess_x = list(zip(*lowess))[0]
    lowess_y = list(zip(*lowess))[1]

    import pylab as pl
    pl.clf()
    pl.plot(x, y, label='y noisy')
    pl.plot(lowess_x, lowess_y, label='y pred')
    pl.legend()
    pl.show()
