# coding: utf-8
"""
    NumpyUtils.py

    Copyright (c) 2016, Masatsuyo Takahashi, KEK-PF
"""

import re
import numpy        as np

def get_valid_index( X ):
    return np.logical_and( np.logical_not( np.isnan( X ) ), np.logical_not( np.isinf( X ) ) )

def _save_names( fh, names, delimiter ):
    line = delimiter.join( names ) + '\n'
    fh.write( bytearray( line, 'cp932' ) )  # utf8 or cp932

def np_savetxt( file, np_array, mode="w", column_names=None ):
    if file.find( '.csv' ) > 0:
        delimiter_ = ','
    elif file.find( '.xlsx' ) > 0:
        assert( mode == "w" )
        np_savexlsx( file, np_array, column_names=column_names )
        return

    elif file.find( '.dat' ) > 0:
        delimiter_ = '\t'

    if mode == 'a':
        with open( file, "ab" ) as fh:
            if column_names is not None:
                _save_names( fh, column_names, delimiter_ )
            np.savetxt( fh, np_array, delimiter=delimiter_ )
    else:
        with open( file, "wb" ) as fh:
            if column_names is not None:
                _save_names( fh, column_names, delimiter_ )
            np.savetxt( fh, np_array, delimiter=delimiter_ )

data_line_re = re.compile( '^\s*\d+' )

def np_loadtxt( fname, usecols=None, delimiter=None, encoding='utf-8', retry=True ):
    global first_data_line

    if type( fname ) == str:
        fh = open( fname, 'r', encoding=encoding )
        is_csv =  fname.find( '.csv' ) > 0
    else:
        fh = fname
        is_csv = False

    comment_lines = []
    first_data_line = None

    def generator( fh ):
        global first_data_line

        for line in fh:
            if type( line ) == bytes:
                line = line.decode()

            if data_line_re.match( line ):
                if first_data_line is None:
                    first_data_line = line
                yield line
            else:
                comment_lines.append( line )

    if delimiter is None:
        delimiter = ',' if is_csv else None

    try:
        array = np.loadtxt( generator( fh ), delimiter=delimiter, usecols=usecols )
    except Exception as e:
        if retry and delimiter is None:
            delim_ = ',' if first_data_line.find( ',' ) > 0 else None
            array, comment_lines = np_loadtxt( fname, delimiter=delim_,
                usecols=usecols, encoding=encoding,
                retry=False     # to avoid infinite recursive calls
                )
        else:
            raise e

    fh.close()

    return array, comment_lines
