# -*- coding: utf-8 -*-
"""

    ファイル名：   TkSupplements.py

    処理内容：

        BlinkingFrame   配下のオブジェクトを点滅できる Frame

    Copyright (c) 2016, Masatsuyo Takahashi

"""
from __future__ import division, print_function, unicode_literals

import os
import sys
if sys.version_info > (3,):
    import tkinter as Tk
    from tkinter import _cnfmerge
else:
    import Tkinter as Tk
    from Tkinter import _cnfmerge

def pad_text( text, length ):
    return text + ' ' * ( length - len(text) )

from BasicUtils import exe_name

"""
    Icon files are supposed to be placed in the same folder,
    named as follows.
        name.ico on Windows
        name.png on Linux
"""

cashed_image = None

def tk_set_icon_portable( window, name=None, use_cashed_image=False ):
    if name is None:
        name = exe_name()
    iconpath = os.path.dirname( os.path.abspath( __file__ ) ) + '/%s' % ( name )
    if use_cashed_image:
        global cashed_image
        if cashed_image is None:
            iconpath += '.png'
            if not os.path.exists(iconpath): return
            print( 'Loading the icon image' )
            cashed_image = Tk.Image( "photo", file=iconpath )
        window.tk.call( 'wm', 'iconphoto', window._w, cashed_image )
    else:
        if os.name == 'nt':         # i.e. on Windows
            iconpath += '.ico'
            if not os.path.exists(iconpath): return
            window.iconbitmap( iconpath )
        else:
            iconpath += '.png'
            if not os.path.exists(iconpath): return
            img = Tk.Image( "photo", file=iconpath )
            window.tk.call( 'wm', 'iconphoto', window._w, img )

"""
    Thanks to: http://stackoverflow.com/questions/21419032/flashing-tkinter-labels
"""
class BlinkingFrame(Tk.Frame):
    def __init__( self, parent, object_spec_array, grid=False, start_proc=None, stop_proc=None, debug=False ):
        Tk.Frame.__init__(self, parent)

        self.objects = []
        for object_spec in object_spec_array:
            obj, pack_info  = object_spec
            self.objects.append( obj )
            if grid:
                obj.grid( **pack_info )
            else:
                obj.pack( **pack_info )

        self.start_proc = start_proc
        self.stop_proc  = stop_proc
        self.switch     = False
        self.reverse    = False
        self.debug      = debug

    def flash(self):
        if not self.reverse and not self.switch:
            return

        for obj in self.objects:
            try:
                bg = obj.cget("background")
                fg = obj.cget("foreground")
                obj.configure(background=fg, foreground=bg)
            except:
                continue

        self.reverse = not self.reverse
        self.after(250, self.flash)

    def start(self):
        if not self.switch:
            if self.debug:
                print( 'start blinking' )
            self.switch = True
            if self.start_proc:
                self.start_proc()
            self.flash()

    def stop(self):
        if self.switch:
            if self.debug:
                print( 'stop blinking' )
            self.switch = False
            if self.stop_proc:
                self.stop_proc()

    def is_blinking( self ):
        return self.switch

"""
    pixel-sizable button widget
    # cf. http://www-acc.kek.jp/kekb/control/Activity/Python/TkIntro/introduction/button.htm
    TODO: doesn't seem to work when placed by grid
"""
class SlimButton(Tk.Frame):
    def __init__( self, parent, text, command, state='normal', height=26, width=60 ):
        Tk.Frame.__init__( self, parent, height=height, width=width )
        self.button = Tk.Button( self, text=text, command=command, state=state )
        self.button.pack( fill=Tk.BOTH, expand=1 )

    def pack( self, cnf={}, **kw ):
        cnf = _cnfmerge((cnf, kw))      # cf. tkinter.dialog.py
        Tk.Frame.pack( self, cnf )
        self.pack_propagate( 0 )        # don't shrink

    def invoke( self ):
        # TODO: other args?
        self.button.invoke()

    def configure( self, cnf={}, **kw ):
        cnf = _cnfmerge((cnf, kw))      # cf. tkinter.dialog.py
        self.button.configure( cnf )

if __name__ == "__main__":
    root = Tk.Tk()

    objects = []
    objects.append( [ Tk.Label( root, text="Hello, world" ), { "fill":"both", "expand":True} ] )

    bf = BlinkingFrame( root, objects )
    Tk.Button( text="Start", command=lambda: bf.start() ).pack()
    Tk.Button( text="Stop", command=lambda: bf.stop() ).pack()

    root.mainloop()
    root.destroy()
